# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import

import numpy as np

import CoolProp
from .Common import process_fluid_state
from .SimpleCycles import BaseCycle, StateContainer


class BasePowerCycle(BaseCycle):
    """A thermodynamic cycle for power producing processes.

    Defines the basic properties and methods to unify access to
    power cycle-related quantities.
    """

    def __init__(self, fluid_ref='HEOS::Water', graph_type='TS', **kwargs):
        """see :class:`CoolProp.Plots.SimpleCycles.BaseCycle` for details."""
        BaseCycle.__init__(self, fluid_ref, graph_type, **kwargs)

    def eta_carnot(self):
        """Carnot efficiency

        Calculates the Carnot efficiency for the specified process, :math:`\eta_c = 1 - \frac{T_c}{T_h}`.

        Returns
        -------
        float
        """
        Tvector = self._cycle_states.T
        return 1. - np.min(Tvector) / np.max(Tvector)

    def eta_thermal(self):
        """Thermal efficiency

        The thermal efficiency for the specified process(es), :math:`\eta_{th} = \frac{\dot{W}_{exp} - \dot{W}_{pum}}{\dot{Q}_{in}}`.

        Returns
        -------
        float
        """
        raise NotImplementedError("Implement it in the subclass.")


class SimpleRankineCycle(BasePowerCycle):
    """A simple Rankine cycle *without* regeneration"""
    STATECOUNT = 4
    STATECHANGE = [
      lambda inp: BaseCycle.state_change(inp, 'S', 'P', 0, ty1='log', ty2='log'),  # Pumping process
      lambda inp: BaseCycle.state_change(inp, 'H', 'P', 1, ty1='lin', ty2='lin'),  # Heat addition
      lambda inp: BaseCycle.state_change(inp, 'H', 'P', 2, ty1='log', ty2='log'),  # Expansion
      lambda inp: BaseCycle.state_change(inp, 'H', 'P', 3, ty1='lin', ty2='lin')  # Heat removal
      ]

    def __init__(self, fluid_ref='HEOS::Water', graph_type='TS', **kwargs):
        """see :class:`CoolProp.Plots.SimpleCycles.BasePowerCycle` for details."""
        BasePowerCycle.__init__(self, fluid_ref, graph_type, **kwargs)

    def simple_solve(self, T0, p0, T2, p2, eta_exp, eta_pum, fluid=None, SI=True):
        """"
        A simple Rankine cycle calculation

        Parameters
        ----------
        T0 : float
            The coldest point, before the pump
        p0 : float
            The lowest pressure, before the pump
        T2 : float
            The hottest point, before the expander
        p2 : float
            The highest pressure, before the expander
        eta_exp : float
            Isentropic expander efficiency
        eta_pum : float
            Isentropic pump efficiency

        Examples
        --------
        >>> import CoolProp
        >>> from CoolProp.Plots import PropertyPlot
        >>> from CoolProp.Plots import SimpleRankineCycle
        >>> pp = PropertyPlot('HEOS::Water', 'TS', unit_system='EUR')
        >>> pp.calc_isolines(CoolProp.iQ, num=11)
        >>> cycle = SimpleRankineCycle('HEOS::Water', 'TS', unit_system='EUR')
        >>> T0 = 300
        >>> pp.state.update(CoolProp.QT_INPUTS,0.0,T0+15)
        >>> p0 = pp.state.keyed_output(CoolProp.iP)
        >>> T2 = 700
        >>> pp.state.update(CoolProp.QT_INPUTS,1.0,T2-150)
        >>> p2 = pp.state.keyed_output(CoolProp.iP)
        >>> cycle.simple_solve(T0, p0, T2, p2, 0.7, 0.8, SI=True)
        >>> cycle.steps = 50
        >>> sc = cycle.get_state_changes()
        >>> import matplotlib.pyplot as plt
        >>> plt.close(cycle.figure)
        >>> pp.draw_process(sc)

        """
        if fluid is not None: self.state = process_fluid_state(fluid)
        if self._state is None:
            raise ValueError("You have to specify a fluid before you can calculate.")

        cycle_states = StateContainer(unit_system=self._system)

        if not SI:
            Tc = self._system[CoolProp.iT].to_SI
            pc = self._system[CoolProp.iP].to_SI
            T0 = Tc(T0)
            p0 = pc(p0)
            T2 = Tc(T2)
            p2 = pc(p2)

        # Subcooled liquid
        self.state.update(CoolProp.PT_INPUTS, p0, T0)
        h0 = self.state.hmass()
        s0 = self.state.smass()
        # Just a showcase for the different accessor methods
        cycle_states[0, 'H'] = h0
        cycle_states[0]['S'] = s0
        cycle_states[0][CoolProp.iP] = p0
        cycle_states[0, CoolProp.iT] = T0

        # Pressurised liquid
        p1 = p2
        self.state.update(CoolProp.PSmass_INPUTS, p1, s0)
        h1 = h0 + (self.state.hmass() - h0) / eta_pum
        self.state.update(CoolProp.HmassP_INPUTS, h1, p1)
        s1 = self.state.smass()
        T1 = self.state.T()
        cycle_states[1, 'H'] = h1
        cycle_states[1, 'S'] = s1
        cycle_states[1, 'P'] = p1
        cycle_states[1, 'T'] = T1

        # Evaporated vapour
        self.state.update(CoolProp.PT_INPUTS, p2, T2)
        h2 = self.state.hmass()
        s2 = self.state.smass()
        cycle_states[2, 'H'] = h2
        cycle_states[2, 'S'] = s2
        cycle_states[2, 'P'] = p2
        cycle_states[2, 'T'] = T2

        # Expanded gas
        p3 = p0
        self.state.update(CoolProp.PSmass_INPUTS, p3, s2)
        h3 = h2 - eta_exp * (h2 - self.state.hmass())
        self.state.update(CoolProp.HmassP_INPUTS, h3, p3)
        s3 = self.state.smass()
        T3 = self.state.T()
        cycle_states[3, 'H'] = h3
        cycle_states[3, 'S'] = s3
        cycle_states[3, 'P'] = p3
        cycle_states[3, 'T'] = T3

        w_net = h2 - h3
        q_boiler = h2 - h1
        eta_th = w_net / q_boiler

        self.cycle_states = cycle_states
        self.fill_states()

    def eta_thermal(self):
        """Thermal efficiency

        The thermal efficiency for the specified process(es), :math:`\eta_{th} = \frac{\dot{W}_{exp} - \dot{W}_{pum}}{\dot{Q}_{in}}`.

        Returns
        -------
        float
        """
        w_net = self.cycle_states[2].H - self.cycle_states[3].H - (self.cycle_states[1].H - self.cycle_states[0].H)
        q_boiler = self.cycle_states[2].H - self.cycle_states[1].H
        return w_net / q_boiler



from __future__ import absolute_import

# If there is a constants.[pyd|so|dylib] in the main directory, it will be imported instead of the constants.py file.
# It should be removed as it is from the older version of CoolProp
from . import constants
if constants.__file__.rsplit('.', 1)[1] not in ['pyc', 'pyo', 'py']:

    import os
    try:
        os.remove(constants.__file__)
        print("constants shared library has been removed.  Please restart your python code")
    except:
        print("Unable to remove" + constants.__file__ + ". Please manually remove it")
    quit()

from .CoolProp import AbstractState
from . import CoolProp
from . import HumidAirProp
from . import State
from .constants import *

__fluids__ = CoolProp.get_global_param_string('fluids_list').split(',')
__incompressibles_pure__ = CoolProp.get_global_param_string('incompressible_list_pure').split(',')
__incompressibles_solution__ = CoolProp.get_global_param_string('incompressible_list_solution').split(',')
__version__ = CoolProp.get_global_param_string('version')
__gitrevision__ = CoolProp.get_global_param_string('gitrevision')

def get(s):
    """
    This is just a shorthand function for getting a parameter from
    ``CoolProp.get_global_param_string``
    """
    return CoolProp.get_global_param_string(s)


def test():
    """
    Run the tests in the test folder
    """
    from .tests import runner
    runner.run()


def get_include_directory():
    """
    Get the include directory for CoolProp header files that are needed if you want
    to compile anything else that uses the CoolProp Cython extension type

    Returns
    -------
    include_directory: The path to the include folder for CoolProp
    """
    import os
    head, file = os.path.split(__file__)
    return os.path.join(head, 'include')


def copy_BibTeX_library(file=None, folder=None):
    """
    Copy the CoolProp BibTeX library file to the file given by ``file``, or the folder given by ``folder``

    If no inputs are provided, the file will be copied to the current working 
    directory

    Parameters
    ----------
    file : string
        Provide if you want to put the file into a given file
    folder : string
        Provide if you want to put the CoolPropBibTeXLibrary.bib file into the given folder

    """
    import os, shutil
    path_to_bib = os.path.join(os.path.split(__file__)[0], 'CoolPropBibTeXLibrary.bib')
    if file is None and folder is None:
        shutil.copy2(path_to_bib, os.path.abspath(os.curdir))
    elif file and folder is None:
        shutil.copy2(path_to_bib, file)
    elif folder and file is None:
        shutil.copy2(path_to_bib, os.path.join(folder, file))
    else:
        raise ValueError('can only provide one of file or folder')



from libcpp cimport bool
from libcpp.string cimport string
from libcpp.vector cimport vector

from . cimport constants_header

from .typedefs cimport CoolPropDbl

cdef extern from "PhaseEnvelope.h" namespace "CoolProp":
    cdef cppclass PhaseEnvelopeData:
        bool TypeI
        size_t iTsat_max, ipsat_max, icrit
        vector[double] T, p, lnT, lnp, rhomolar_liq, rhomolar_vap, lnrhomolar_liq, lnrhomolar_vap, hmolar_liq, hmolar_vap, smolar_liq, smolar_vap, Q
        vector[vector[double]] x, y, K

cdef extern from "DataStructures.h" namespace "CoolProp":
    cdef cppclass CriticalState:
        double T, p, rhomolar, hmolar, smolar
        bool stable

cdef extern from "AbstractState.h" namespace "CoolProp":

    cdef cppclass GuessesStructure:
        double T, p, rhomolar, hmolar, smolar
        double rhomolar_liq, rhomolar_vap
        double _rhomolar_liq, _rhomolar_vap
        vector[double] x, y

    cdef cppclass SpinodalData:
        vector[double] tau, delta, M1

    cdef cppclass AbstractState:

        ## Nullary Constructor
        AbstractState() except +ValueError

        ## Constructor with fluid name
        AbstractState(string FluidName) except +ValueError

        void set_mole_fractions(vector[double]) except+ValueError
        void set_mass_fractions(vector[double]) except+ValueError
        void set_volu_fractions(vector[double]) except+ValueError

        vector[CoolPropDbl] mole_fractions_liquid() except +ValueError
        vector[CoolPropDbl] mole_fractions_vapor() except +ValueError
        vector[CoolPropDbl] get_mole_fractions() except +ValueError
        vector[CoolPropDbl] get_mass_fractions() except +ValueError

        constants_header.phases phase() except +ValueError
        void specify_phase(constants_header.phases phase) except +ValueError
        void unspecify_phase() except +ValueError

        void change_EOS(const size_t, const string &) except +ValueError

        void set_binary_interaction_double(const string, const string &, const string &, const double s) except +ValueError
        void set_binary_interaction_string(const string &, const string &, const string &, const string &) except +ValueError
        double get_binary_interaction_double(const string &, const string &, const string &) except +ValueError
        string get_binary_interaction_string(const string &, const string &, const string &) except +ValueError
        void set_binary_interaction_string(const size_t, const size_t, const string &, const string &) except +ValueError
        void apply_simple_mixing_rule(size_t, size_t, const string &) except +ValueError

        double get_binary_interaction_double(const size_t, const size_t, const string &) except +ValueError
        void set_binary_interaction_double(const size_t, const size_t, const string &, const double s) except +ValueError

        string name() except +ValueError
        string backend_name() except +ValueError
        vector[string] fluid_names() except +ValueError
        string fluid_param_string(const string &) except +ValueError
        void set_fluid_parameter_double(const size_t, const string&, const double) except +ValueError
        double get_fluid_parameter_double(const size_t, const string&) except +ValueError

        bool clear()

        ## Limits
        double Tmin() except +ValueError
        double Tmax() except +ValueError
        double pmax() except +ValueError
        double Ttriple() except +ValueError

        ## Critical point
        double T_critical() except +ValueError
        double rhomass_critical() except +ValueError
        double rhomolar_critical() except +ValueError
        double p_critical() except +ValueError
        vector[CriticalState] all_critical_points() except +ValueError
        void criticality_contour_values(double &L1star, double &M1star) except +ValueError

        ## Spinodal curve
        void build_spinodal() except +ValueError
        SpinodalData get_spinodal_data() except +ValueError

        ## Tangent plane analysis
        double tangent_plane_distance(const double, const double, const vector[double], const double) except +ValueError

        ## Reducing point
        double T_reducing() except +ValueError
        double rhomolar_reducing() except +ValueError
        double rhomass_reducing() except +ValueError

        void ideal_curve(const string &, vector[double] &T, vector[double] &p) except +ValueError

        ## Property updater
        ## Uses the indices in CoolProp for the input parameters
        void update(constants_header.input_pairs iInput1, double Value1, double Value2) except +ValueError
        ## Uses the indices in CoolProp for the input parameters
        void update_with_guesses(constants_header.input_pairs iInput1, double Value1, double Value2, GuessesStructure) except +ValueError

        ## Bulk properties accessors - temperature, pressure and density are directly calculated every time
        ## All other parameters are calculated on an as-needed basis
        ## If single-phase, just plug into the EOS, otherwise need to do two-phase analysis
        double T() except +ValueError
        double rhomolar() except +ValueError
        double rhomass() except +ValueError
        double p() except +ValueError
        double compressibility_factor() except +ValueError
        double Q() except +ValueError
        double hmolar() except +ValueError
        double hmass() except +ValueError
        double smolar() except +ValueError
        double smass() except +ValueError
        double umolar() except +ValueError
        double umass() except +ValueError
        double cpmolar() except +ValueError
        double cpmass() except +ValueError
        double cp0molar() except +ValueError
        double cp0mass() except +ValueError
        double cvmolar() except +ValueError
        double cvmass() except +ValueError
        double gibbsmolar() except +ValueError
        double gibbsmass() except +ValueError
        double helmholtzmolar() except +ValueError
        double helmholtzmass() except +ValueError
        double speed_sound() except +ValueError
        double tau() except +ValueError
        double delta() except +ValueError
        double viscosity() except+ValueError
        double conductivity() except+ValueError
        void conformal_state(const string &, CoolPropDbl &, CoolPropDbl &) except +ValueError
        void conductivity_contributions(CoolPropDbl &dilute, CoolPropDbl &initial_density, CoolPropDbl &residual, CoolPropDbl &critical) except +ValueError
        void viscosity_contributions(CoolPropDbl &dilute, CoolPropDbl &initial_density, CoolPropDbl &residual, CoolPropDbl &critical) except +ValueError

        double gibbsmolar_excess() except +ValueError
        double gibbsmass_excess() except +ValueError
        double hmolar_excess() except +ValueError
        double hmass_excess() except +ValueError
        double smolar_excess() except +ValueError
        double smass_excess() except +ValueError
        double umolar_excess() except +ValueError
        double umass_excess() except +ValueError
        double volumemolar_excess() except +ValueError
        double volumemass_excess() except +ValueError
        double helmholtzmolar_excess() except +ValueError
        double helmholtzmass_excess() except +ValueError

        double gibbsmolar_residual() except +ValueError
        double hmolar_residual() except +ValueError
        double smolar_residual() except +ValueError

        double surface_tension() except +ValueError
        double Prandtl() except +ValueError
        double Bvirial() except +ValueError
        double Cvirial() except +ValueError
        double PIP() except +ValueError
        double fundamental_derivative_of_gas_dynamics() except +ValueError
        double isothermal_compressibility() except +ValueError
        double isobaric_expansion_coefficient() except +ValueError
        double fugacity(size_t) except +ValueError
        double fugacity_coefficient(size_t) except +ValueError
        double chemical_potential(size_t) except +ValueError

        double get_fluid_constant(size_t,constants_header.parameters) except+ValueError
        double keyed_output(constants_header.parameters) except+ValueError
        double trivial_keyed_output(constants_header.parameters) except+ValueError
        double saturated_liquid_keyed_output(constants_header.parameters) except+ValueError
        double saturated_vapor_keyed_output(constants_header.parameters) except+ValueError

        double molar_mass() except+ValueError
        double acentric_factor() except+ValueError
        double gas_constant() except+ValueError

        CoolPropDbl first_partial_deriv(constants_header.parameters, constants_header.parameters, constants_header.parameters) except+ValueError
        CoolPropDbl second_partial_deriv(constants_header.parameters, constants_header.parameters, constants_header.parameters, constants_header.parameters, constants_header.parameters) except+ValueError
        CoolPropDbl first_saturation_deriv(constants_header.parameters, constants_header.parameters) except+ValueError
        CoolPropDbl second_saturation_deriv(constants_header.parameters, constants_header.parameters, constants_header.parameters) except+ValueError
        double first_two_phase_deriv(constants_header.parameters Of, constants_header.parameters Wrt, constants_header.parameters Constant) except+ValueError
        double second_two_phase_deriv(constants_header.parameters Of, constants_header.parameters Wrt1, constants_header.parameters Constant1, constants_header.parameters Wrt2, constants_header.parameters Constant2) except+ValueError
        double first_two_phase_deriv_splined(constants_header.parameters Of, constants_header.parameters Wrt, constants_header.parameters Constant, double x_end) except+ValueError
        void true_critical_point(double &T, double &rho) except +ValueError

        double melting_line(int,int,double) except+ValueError
        bool has_melting_line() except+ValueError
        double saturation_ancillary(constants_header.parameters, int, constants_header.parameters, double) except +ValueError

        double build_phase_envelope() except+ValueError
        void build_phase_envelope(string) except+ValueError
        PhaseEnvelopeData get_phase_envelope_data() except+ValueError

        CoolPropDbl alpha0() except+ValueError
        CoolPropDbl dalpha0_dDelta() except+ValueError
        CoolPropDbl dalpha0_dTau() except+ValueError
        CoolPropDbl d2alpha0_dDelta2() except+ValueError
        CoolPropDbl d2alpha0_dDelta_dTau() except+ValueError
        CoolPropDbl d2alpha0_dTau2() except+ValueError
        CoolPropDbl d3alpha0_dTau3() except+ValueError
        CoolPropDbl d3alpha0_dDelta_dTau2() except+ValueError
        CoolPropDbl d3alpha0_dDelta2_dTau() except+ValueError
        CoolPropDbl d3alpha0_dDelta3() except+ValueError

        CoolPropDbl alphar() except+ValueError
        CoolPropDbl dalphar_dDelta() except+ValueError
        CoolPropDbl dalphar_dTau() except+ValueError
        CoolPropDbl d2alphar_dDelta2() except+ValueError
        CoolPropDbl d2alphar_dDelta_dTau() except+ValueError
        CoolPropDbl d2alphar_dTau2() except+ValueError
        CoolPropDbl d3alphar_dDelta3() except+ValueError
        CoolPropDbl d3alphar_dDelta2_dTau() except+ValueError
        CoolPropDbl d3alphar_dDelta_dTau2() except+ValueError
        CoolPropDbl d3alphar_dTau3() except+ValueError
        CoolPropDbl d4alphar_dDelta4() except+ValueError
        CoolPropDbl d4alphar_dDelta3_dTau() except+ValueError
        CoolPropDbl d4alphar_dDelta2_dTau2() except+ValueError
        CoolPropDbl d4alphar_dDelta_dTau3() except+ValueError
        CoolPropDbl d4alphar_dTau4() except+ValueError


# The static factory method for the AbstractState
cdef extern from "AbstractState.h" namespace "CoolProp::AbstractState":
    AbstractState* factory(const string &backend, const string &fluid_string) except+ValueError



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import

import numpy as np

import CoolProp
from .Common import process_fluid_state
from .SimpleCycles import BaseCycle, StateContainer


class BaseCompressionCycle(BaseCycle):
    """A thermodynamic cycle for vapour compression processes.

    Defines the basic properties and methods to unify access to
    compression cycle-related quantities.
    """

    def __init__(self, fluid_ref='HEOS::Water', graph_type='PH', **kwargs):
        """see :class:`CoolProp.Plots.SimpleCycles.BaseCycle` for details."""
        BaseCycle.__init__(self, fluid_ref, graph_type, **kwargs)

    def eta_carnot_heating(self):
        """Carnot efficiency

        Calculates the Carnot efficiency for a heating process, :math:`\eta_c = \frac{T_h}{T_h-T_c}`.

        Returns
        -------
        float
        """
        Tvector = self._cycle_states.T
        return np.max(Tvector) / (np.max(Tvector) - np.min(Tvector))

    def eta_carnot_cooling(self):
        """Carnot efficiency

        Calculates the Carnot efficiency for a cooling process, :math:`\eta_c = \frac{T_c}{T_h-T_c}`.

        Returns
        -------
        float
        """
        Tvector = self._cycle_states.T
        return np.min(Tvector) / (np.max(Tvector) - np.min(Tvector))


class SimpleCompressionCycle(BaseCompressionCycle):
    """A simple vapour compression cycle"""
    STATECOUNT = 4
    STATECHANGE = [
      lambda inp: BaseCycle.state_change(inp, 'S', 'P', 0, ty1='log', ty2='log'),  # Compression process
      lambda inp: BaseCycle.state_change(inp, 'H', 'P', 1, ty1='lin', ty2='lin'),  # Heat removal
      lambda inp: BaseCycle.state_change(inp, 'H', 'P', 2, ty1='log', ty2='log'),  # Expansion
      lambda inp: BaseCycle.state_change(inp, 'H', 'P', 3, ty1='lin', ty2='lin')  # Heat addition
      ]

    def __init__(self, fluid_ref='HEOS::Water', graph_type='PH', **kwargs):
        """see :class:`CoolProp.Plots.SimpleCyclesCompression.BaseCompressionCycle` for details."""
        BaseCompressionCycle.__init__(self, fluid_ref, graph_type, **kwargs)

    def simple_solve(self, T0, p0, T2, p2, eta_com, fluid=None, SI=True):
        """"
        A simple vapour compression cycle calculation

        Parameters
        ----------
        T0 : float
            The evaporated fluid, before the compressor
        p0 : float
            The evaporated fluid, before the compressor
        T2 : float
            The condensed fluid, before the expansion valve
        p2 : float
            The condensed fluid, before the expansion valve
        eta_com : float
            Isentropic compressor efficiency

        Examples
        --------
        >>> import CoolProp
        >>> from CoolProp.Plots import PropertyPlot
        >>> from CoolProp.Plots import SimpleCompressionCycle
        >>> pp = PropertyPlot('HEOS::R134a', 'PH', unit_system='EUR')
        >>> pp.calc_isolines(CoolProp.iQ, num=11)
        >>> cycle = SimpleCompressionCycle('HEOS::R134a', 'PH', unit_system='EUR')
        >>> T0 = 280
        >>> pp.state.update(CoolProp.QT_INPUTS,0.0,T0-15)
        >>> p0 = pp.state.keyed_output(CoolProp.iP)
        >>> T2 = 310
        >>> pp.state.update(CoolProp.QT_INPUTS,1.0,T2+10)
        >>> p2 = pp.state.keyed_output(CoolProp.iP)
        >>> cycle.simple_solve(T0, p0, T2, p2, 0.7, SI=True)
        >>> cycle.steps = 50
        >>> sc = cycle.get_state_changes()
        >>> import matplotlib.pyplot as plt
        >>> plt.close(cycle.figure)
        >>> pp.draw_process(sc)

        """
        if fluid is not None:
            self.state = process_fluid_state(fluid)
        if self._state is None:
            raise ValueError("You have to specify a fluid before you can calculate.")

        cycle_states = StateContainer(unit_system=self._system)

        if not SI:
            conv_t = self._system[CoolProp.iT].to_SI
            conv_p = self._system[CoolProp.iP].to_SI
            T0 = conv_t(T0)
            p0 = conv_p(p0)
            T2 = conv_t(T2)
            p2 = conv_p(p2)

        # Gas from evaporator
        self.state.update(CoolProp.PT_INPUTS, p0, T0)
        h0 = self.state.hmass()
        s0 = self.state.smass()
        # Just a showcase for the different accessor methods
        cycle_states[0, 'H'] = h0
        cycle_states[0]['S'] = s0
        cycle_states[0][CoolProp.iP] = p0
        cycle_states[0, CoolProp.iT] = T0

        # Pressurised vapour
        p1 = p2
        self.state.update(CoolProp.PSmass_INPUTS, p1, s0)
        h1 = h0 + (self.state.hmass() - h0) / eta_com
        self.state.update(CoolProp.HmassP_INPUTS, h1, p1)
        s1 = self.state.smass()
        T1 = self.state.T()
        cycle_states[1, 'H'] = h1
        cycle_states[1, 'S'] = s1
        cycle_states[1, 'P'] = p1
        cycle_states[1, 'T'] = T1

        # Condensed vapour
        self.state.update(CoolProp.PT_INPUTS, p2, T2)
        h2 = self.state.hmass()
        s2 = self.state.smass()
        cycle_states[2, 'H'] = h2
        cycle_states[2, 'S'] = s2
        cycle_states[2, 'P'] = p2
        cycle_states[2, 'T'] = T2

        # Expanded fluid, 2-phase
        p3 = p0
        h3 = h2
        self.state.update(CoolProp.HmassP_INPUTS, h3, p3)
        s3 = self.state.smass()
        T3 = self.state.T()
        cycle_states[3, 'H'] = h3
        cycle_states[3, 'S'] = s3
        cycle_states[3, 'P'] = p3
        cycle_states[3, 'T'] = T3

        w_net = h0 - h1
        q_evap = h0 - h3

        self.cycle_states = cycle_states
        self.fill_states()

    def simple_solve_dt(self, Te, Tc, dT_sh, dT_sc, eta_com, fluid=None, SI=True):
        """"
        A simple vapour compression cycle calculation based on
        superheat, subcooling and temperatures.

        Parameters
        ----------
        Te : float
            The evaporation temperature
        Tc : float
            The condensation temperature
        dT_sh : float
            The superheat after the evaporator
        dT_sc : float
            The subcooling after the condenser
        eta_com : float
            Isentropic compressor efficiency

        Examples
        --------
        >>> import CoolProp
        >>> from CoolProp.Plots import PropertyPlot
        >>> from CoolProp.Plots import SimpleCompressionCycle
        >>> pp = PropertyPlot('HEOS::R134a', 'PH', unit_system='EUR')
        >>> pp.calc_isolines(CoolProp.iQ, num=11)
        >>> cycle = SimpleCompressionCycle('HEOS::R134a', 'PH', unit_system='EUR')
        >>> Te = 265
        >>> Tc = 300
        >>> cycle.simple_solve_dt(Te, Tc, 10, 15, 0.7, SI=True)
        >>> cycle.steps = 50
        >>> sc = cycle.get_state_changes()
        >>> import matplotlib.pyplot as plt
        >>> plt.close(cycle.figure)
        >>> pp.draw_process(sc)
        """
        if fluid is not None:
            self.state = process_fluid_state(fluid)
        if self._state is None:
            raise ValueError("You have to specify a fluid before you can calculate.")

        if not SI:
            conv_t = self._system[CoolProp.iT].to_SI
            Te = conv_p(Te)
            Tc = conv_p(Tc)

        # Get the saturation conditions
        self.state.update(CoolProp.QT_INPUTS, 1.0, Te)
        p0 = self.state.p()
        self.state.update(CoolProp.QT_INPUTS, 0.0, Tc)
        p2 = self.state.p()

        T0 = Te + dT_sh
        T2 = Tc - dT_sc

        self.simple_solve(T0, p0, T2, p2, eta_com, fluid=None, SI=True)

    def COP_heating(self):
        """COP for a heating process

        Calculates the coefficient of performance for a heating process, :math:`COP_h = \frac{q_{con}}{w_{comp}}`.

        Returns
        -------
        float
        """
        return (self.cycle_states[1, 'H'] - self.cycle_states[2, 'H']) / (self.cycle_states[1, 'H'] - self.cycle_states[0, 'H'])

    def COP_cooling(self):
        """COP for a cooling process

        Calculates the coefficient of performance for a cooling process, :math:`COP_c = \frac{q_{eva}}{w_{comp}}`.

        Returns
        -------
        float
        """
        return (self.cycle_states[0, 'H'] - self.cycle_states[3, 'H']) / (self.cycle_states[1, 'H'] - self.cycle_states[0, 'H'])



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import cPickle
from ConfigParser import ConfigParser

import numpy as np

from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg
from pylab import Figure

from CoolProp.HumidAirProp import HAProps, HAProps_Aux

from PyQt4.QtGui import (QDialog, QGridLayout, QProgressBar, QLabel,
                         QDialogButtonBox, QPushButton, QFileDialog, QApplication)

Preferences = ConfigParser()
config_path = os.path.join(os.path.dirname(__file__), "psyrc")
Preferences.read(config_path)
P = Preferences.getfloat("General", "P")


def _Pbar(Z):
    """
    ASHRAE Fundamentals Handbook pag 1.1 eq. 3
    input:
        Z: altitude, m
    return
        standard atmosphere barometric pressure, Pa
    """
    return 101325. * (1 - 2.25577e-5 * Z)**5.256


class PsychroPlot(FigureCanvasQTAgg):
    """
    Plot widget for psychrometric chart
        Add custom margins
        Define a pointer to text state properties, to remove and redraw
    """

    def __init__(self, parent=None, width=15, height=5, dpi=100):
        self.fig = Figure(figsize=(width, height), dpi=dpi)
        FigureCanvasQTAgg.__init__(self, self.fig)
        self.setParent(parent)
        self.axes2D = self.fig.add_subplot(111)
        FigureCanvasQTAgg.updateGeometry(self)
        self.axes2D.figure.subplots_adjust(left=0.01, right=0.92,
                                           bottom=0.05, top=0.98)
        self.notes = []

    def plot(self, *args, **kwargs):
        self.axes2D.plot(*args, **kwargs)

    def config(self):
        self.axes2D.set_autoscale_on(False)
        self.axes2D.set_xlabel(u"Tdb, ºC")
        self.axes2D.set_ylabel("Absolute humidity, kg/kg")
        self.axes2D.yaxis.set_ticks_position("right")
        self.axes2D.yaxis.set_label_position("right")

        tmin = Preferences.getfloat("Psychr", "isotdbStart") - 273.15
        tmax = Preferences.getfloat("Psychr", "isotdbEnd") - 273.15

        self.axes2D.set_xlim(tmin, tmax)
        self.axes2D.set_ylim(0, 0.04)

    def showPointData(self, state):
        self.clearPointData()

        yi = 0.99
        keys = "tdb", "tdp", "twb", "HR", "w", "h", "v", "rho"
        units = u"ºC", u"ºC", u"ºC", "%", "kgw/kgda", "kJ/kg", u"m³/kg", u"kg/m³"
        for key, txt in zip(keys, units):
            self.notes.append(self.axes2D.annotate(
                "%s: %0.4f %s" % (key, state.__getattribute__(key), txt), (0.01, yi),
                xycoords='axes fraction', size="small", va="center"))
            yi -= 0.025
        self.draw()

    def clearPointData(self):
        while self.notes:
            anotation = self.notes.pop()
            anotation.remove()
        self.draw()


class PsyCoolprop(object):
    """
    Psychrometric state using coolprop library

    kwargs definition parameters:
        P: Pressure, Pa
        z: altitude, m

        tdp: dew-point temperature
        tdb: dry-bulb temperature
        twb: web-bulb temperature
        w: Humidity Ratio [kg water/kg dry air]
        HR: Relative humidity
        h: Mixture enthalpy
        v: Mixture specified volume

    P: mandatory input for barometric pressure, z is an alternate pressure input
    it needs other two input parameters:
        0 - tdb, w
        1 - tdb, HR
        2 - tdb, twb
        3 - tdb, tdp
        4 - tdp, HR
        5 - tdp, twb
        6 - twb, w
    """
    kwargs = {"z": 0.0,
              "P": 0.0,

              "tdb": 0.0,
              "tdb": 0.0,
              "twb": 0.0,
              "w": None,
              "HR": None,
              "h": None,
              "v": 0.0}
    status = 0
    msg = "Unknown variables"

    def __init__(self, **kwargs):
        self.kwargs = self.__class__.kwargs.copy()
        self.__call__(**kwargs)

    def __call__(self, **kwargs):
        self.kwargs.update(kwargs)

        if self.calculable:
            self.status = 1
            self.calculo()
            self.msg = "Solved"

    @property
    def calculable(self):
        tdp = self.kwargs.get("tdp", 0)
        tdb = self.kwargs.get("tdb", 0)
        twb = self.kwargs.get("twb", 0)
        w = self.kwargs.get("w", None)
        HR = self.kwargs.get("HR", None)
        h = self.kwargs.get("h", None)
        v = self.kwargs.get("v", 0)

        self._mode = 0
        if tdb and w is not None:
            self._mode = ("Tdb", "W")
        elif tdb and HR is not None:
            self._mode = ("Tdb", "RH")
        elif tdb and twb:
            self._mode = ("Tdb", "Twb")
        elif tdb and tdp:
            self._mode = ("Tdb", "Tdp")
        elif tdp and HR is not None:
            self._mode = ("Tdp", "RH")

        return bool(self._mode)

    def calculo(self):
        tdp, tdb, twb, P, Pvs, Pv, ws, w, HR, v, h = self._lib()
        self.tdp = tdp - 273.15
        self.tdb = tdb - 273.15
        self.twb = twb - 273.15
        self.P = P
        self.Pvs = Pvs
        self.Pv = Pv
        self.ws = ws
        self.w = w
        self.HR = HR
        self.mu = w / ws * 100
        self.v = v
        self.rho = 1 / v
        self.h = h
        self.Xa = 1 / (1 + self.w / 0.62198)
        self.Xw = 1 - self.Xa

    def args(self):
        # Correct coolprop custom namespace versus pychemqt namespace
        if "Tdb" in self._mode:
            self.kwargs["Tdb"] = self.kwargs["tdb"]
        if "Twb" in self._mode:
            self.kwargs["Twb"] = self.kwargs["twb"]
        if "Tdp" in self._mode:
            self.kwargs["Tdp"] = self.kwargs["tdp"]
        if "RH" in self._mode:
            self.kwargs["RH"] = self.kwargs["HR"]
        if "W" in self._mode:
            self.kwargs["W"] = self.kwargs["w"]

        var1 = self.kwargs[self._mode[0]]
        var2 = self.kwargs[self._mode[1]]

        # units conversion to coolprop expected unit:
        # HR in 0-1, H in kJ/kg, S in kJ/kgK
        if "RH" in self._mode[0]:
            var1 /= 100.
        if "RH" in self._mode[1]:
            var2 /= 100.

        args = ("P", self._P_kPa, self._mode[0], var1, self._mode[1], var2)
        return args

    def _P(self):
        """Barometric pressure calculation, Pa"""
        if self.kwargs["P"]:
            P = self.kwargs["P"]
        elif self.kwargs["z"]:
            P = _Pbar(self.kwargs["z"])
        else:
            P = 101325.
        return P

    @property
    def _P_kPa(self):
        """Property for ease access to pressure in kPa"""
        P = self._P()
        return P / 1000.

    def _lib(self):
        args = self.args()
        P = self._P()

        if "Tdb" in self._mode:
            tdb = self.kwargs["Tdb"]
        else:
            tdb = HAProps("Tdb", *args)
        tdp = HAProps("Tdp", *args)
        twb = HAProps("Twb", *args)
        w = HAProps("W", *args)
        HR = HAProps("RH", *args) * 100
        Pvs = HAProps_Aux("p_ws", tdb, self._P_kPa, w)[0] * 1000
        Pv = Pvs * HR / 100
        ws = HAProps("W", "P", self._P_kPa, "Tdb", tdb, "RH", 1)
        v = HAProps("V", *args)
        h = HAProps("H", *args)

        return tdp, tdb, twb, P, Pvs, Pv, ws, w, HR, v, h

    @classmethod
    def calculatePlot(cls, parent):
        """Function to calculate points in chart"""

        data = {}
        P = Preferences.getfloat("General", "P")
        P_kPa = P / 1000
        t = cls.LineList("isotdb", Preferences)

        # Saturation line
        Hs = []
        for tdb in t:
            Hs.append(HAProps("W", "P", P_kPa, "Tdb", tdb, "RH", 1))
            parent.progressBar.setValue(5 * len(Hs) / len(t))
        data["t"] = t
        data["Hs"] = Hs

        # left limit of isow lines
        H = cls.LineList("isow", Preferences)
        th = []
        for w in H:
            if w:
                tdp = HAProps("Tdp", "P", 101.325, "W", w, "RH", 1)
                th.append(tdp - 273.15)
            else:
                tmin = Preferences.getfloat("Psychr", "isotdbStart")
                th.append(tmin - 273.15)
        data["H"] = H
        data["th"] = th

        # Humidity ratio lines
        hr = cls.LineList("isohr", Preferences)
        Hr = {}
        cont = 0
        for i in hr:
            Hr[i] = []
            for tdb in t:
                Hr[i].append(HAProps("W", "P", P_kPa, "Tdb", tdb, "RH", i / 100.))
                cont += 1
                parent.progressBar.setValue(5 + 10 * cont / len(hr) / len(Hs))
        data["Hr"] = Hr

        # Twb
        lines = cls.LineList("isotwb", Preferences)
        Twb = {}
        cont = 0
        for T in lines:
            ws = HAProps("W", "P", P_kPa, "RH", 1, "Tdb", T)
            H = [ws, 0]
            Tw = [T - 273.15, HAProps("Tdb", "P", P_kPa, "Twb", T, "RH", 0) - 273.15]
            cont += 1
            parent.progressBar.setValue(15 + 75 * cont / len(lines))
            Twb[T] = (H, Tw)
        data["Twb"] = Twb

        # v
        lines = cls.LineList("isochor", Preferences)
        V = {}
        rh = np.arange(1, -0.05, -0.05)
        for cont, v in enumerate(lines):
            w = []
            Td = []
            for r in rh:
                w.append(HAProps("W", "P", P_kPa, "RH", r, "V", v))
                Td.append(HAProps("Tdb", "P", P_kPa, "RH", r, "V", v) - 273.15)
            parent.progressBar.setValue(90 + 10 * cont / len(lines))
            V[v] = (Td, w)
        data["v"] = V

        return data

    @staticmethod
    def LineList(name, Preferences):
        """Return a list with the values of isoline name to plot"""
        if Preferences.getboolean("Psychr", name + "Custom"):
            t = []
            for i in Preferences.get("Psychr", name + 'List').split(','):
                if i:
                    t.append(float(i))
        else:
            start = Preferences.getfloat("Psychr", name + "Start")
            end = Preferences.getfloat("Psychr", name + "End")
            step = Preferences.getfloat("Psychr", name + "Step")
            t = np.arange(start, end, step)
        return t


class UI_Psychrometry(QDialog):
    """Psychrometric charts tool"""

    def __init__(self, parent=None):
        super(UI_Psychrometry, self).__init__(parent)
        self.showMaximized()
        self.setWindowTitle("Psychrometric chart")

        layout = QGridLayout(self)
        self.diagrama2D = PsychroPlot(self, dpi=90)
        self.diagrama2D.fig.canvas.mpl_connect('motion_notify_event', self.scroll)
        layout.addWidget(self.diagrama2D, 1, 1, 1, 2)
        self.progressBar = QProgressBar()
        self.progressBar.setVisible(False)
        layout.addWidget(self.progressBar, 2, 1)
        self.status = QLabel()
        layout.addWidget(self.status, 2, 1)

        self.buttonBox = QDialogButtonBox(QDialogButtonBox.Close)
        butonPNG = QPushButton("Save as PNG")
        self.buttonBox.addButton(butonPNG, QDialogButtonBox.AcceptRole)
        self.buttonBox.rejected.connect(self.reject)
        self.buttonBox.accepted.connect(self.savePNG)
        layout.addWidget(self.buttonBox, 2, 2)

        self.plot()

    def savePNG(self):
        """Save chart image to png file"""
        fname = unicode(QFileDialog.getSaveFileName(
            self, "Save chart to file",
            "./", "Portable Network Graphics (*.png)"))
        self.diagrama2D.fig.savefig(fname, facecolor='#eeeeee')

    def drawlabel(self, name, Preferences, t, W, label, unit):
        """
        Draw annotation for isolines
            name: name of isoline
            Preferences: Configparse instance of pychemqt preferences
            t: x array of line
            W: y array of line
            label: text value to draw
            unit: text units to draw
        """
        if Preferences.getboolean("Psychr", name + "label"):
            tmin = Preferences.getfloat("Psychr", "isotdbStart") - 273.15
            tmax = Preferences.getfloat("Psychr", "isotdbEnd") - 273.15
            x = tmax - tmin
            wmin = Preferences.getfloat("Psychr", "isowStart")
            wmax = Preferences.getfloat("Psychr", "isowEnd")
            y = wmax - wmin

            i = 0
            for ti, wi in zip(t, W):
                if tmin <= ti <= tmax and wmin <= wi <= wmax:
                    i += 1
            label = str(label)
            if Preferences.getboolean("Psychr", name + "units"):
                label += unit
            pos = Preferences.getfloat("Psychr", name + "position")
            p = int(i * pos / 100 - 1)
            rot = np.arctan((W[p] - W[p - 1]) / y / (t[p] - t[p - 1]) * x) * 360.0 / 2.0 / np.pi
            self.diagrama2D.axes2D.annotate(label, (t[p], W[p]),
                rotation=rot, size="small", ha="center", va="center")

    def plot(self):
        """Plot chart"""
        Preferences = ConfigParser()
        Preferences.read("psyrc")

        self.diagrama2D.axes2D.clear()
        self.diagrama2D.config()
        filename = "%i.pkl" % P
        if os.path.isfile(filename):
            with open(filename, "r") as archivo:
                data = cPickle.load(archivo)
                self.status.setText("Loading cached data...")
                QApplication.processEvents()
        else:
            self.progressBar.setVisible(True)
            self.status.setText("Calculating data, be patient...")
            QApplication.processEvents()
            data = PsyCoolprop.calculatePlot(self)
            cPickle.dump(data, open(filename, "w"))
            self.progressBar.setVisible(False)
        self.status.setText("Plotting...")
        QApplication.processEvents()

        tmax = Preferences.getfloat("Psychr", "isotdbEnd") - 273.15

        t = [ti - 273.15 for ti in data["t"]]
        Hs = data["Hs"]
        format = {}
        format["ls"] = Preferences.get("Psychr", "saturationlineStyle")
        format["lw"] = Preferences.getfloat("Psychr", "saturationlineWidth")
        format["color"] = Preferences.get("Psychr", "saturationColor")
        format["marker"] = Preferences.get("Psychr", "saturationmarker")
        format["markersize"] = 3
        self.diagrama2D.plot(t, Hs, **format)

        format = {}
        format["ls"] = Preferences.get("Psychr", "isotdblineStyle")
        format["lw"] = Preferences.getfloat("Psychr", "isotdblineWidth")
        format["color"] = Preferences.get("Psychr", "isotdbColor")
        format["marker"] = Preferences.get("Psychr", "isotdbmarker")
        format["markersize"] = 3
        for i, T in enumerate(t):
            self.diagrama2D.plot([T, T], [0, Hs[i]], **format)

        H = data["H"]
        th = data["th"]
        format = {}
        format["ls"] = Preferences.get("Psychr", "isowlineStyle")
        format["lw"] = Preferences.getfloat("Psychr", "isowlineWidth")
        format["color"] = Preferences.get("Psychr", "isowColor")
        format["marker"] = Preferences.get("Psychr", "isowmarker")
        format["markersize"] = 3
        for i, H in enumerate(H):
            self.diagrama2D.plot([th[i], tmax], [H, H], **format)

        format = {}
        format["ls"] = Preferences.get("Psychr", "isohrlineStyle")
        format["lw"] = Preferences.getfloat("Psychr", "isohrlineWidth")
        format["color"] = Preferences.get("Psychr", "isohrColor")
        format["marker"] = Preferences.get("Psychr", "isohrmarker")
        format["markersize"] = 3
        for Hr, H0 in data["Hr"].iteritems():
            self.diagrama2D.plot(t, H0, **format)
            self.drawlabel("isohr", Preferences, t, H0, Hr, "%")

        format = {}
        format["ls"] = Preferences.get("Psychr", "isotwblineStyle")
        format["lw"] = Preferences.getfloat("Psychr", "isotwblineWidth")
        format["color"] = Preferences.get("Psychr", "isotwbColor")
        format["marker"] = Preferences.get("Psychr", "isotwbmarker")
        format["markersize"] = 3
        for T, (H, Tw) in data["Twb"].iteritems():
            self.diagrama2D.plot(Tw, H, **format)
            value = T - 273.15
            txt = u"ºC"
            self.drawlabel("isotwb", Preferences, Tw, H, value, txt)

        format = {}
        format["ls"] = Preferences.get("Psychr", "isochorlineStyle")
        format["lw"] = Preferences.getfloat("Psychr", "isochorlineWidth")
        format["color"] = Preferences.get("Psychr", "isochorColor")
        format["marker"] = Preferences.get("Psychr", "isochormarker")
        format["markersize"] = 3
        for v, (Td, H) in data["v"].iteritems():
            self.diagrama2D.plot(Td, H, **format)
            value = v
            txt = u"m³/kg"
            self.drawlabel("isochor", Preferences, Td, H, value, txt)

        self.diagrama2D.draw()
        self.status.setText("P = %i Pa" % P)

    def scroll(self, event):
        """Update graph annotate when mouse scroll over chart"""
        if event.xdata and event.ydata:
            punto = self.createState(event.xdata, event.ydata)
            if event.ydata < punto.ws:
                self.diagrama2D.showPointData(punto)
            else:
                self.diagrama2D.clearPointData()

    def createState(self, x, y):
        """Create psychrometric state from click or mouse position"""
        tdb = x + 273.15
        punto = PsyCoolprop(P=P, tdb=tdb, w=y)
        return punto


if __name__ == "__main__":
    import sys
    app = QApplication(sys.argv)
    aireHumedo = UI_Psychrometry()
    aireHumedo.show()
    sys.exit(app.exec_())




# This file was auto-generated by the PsychChart.py script in wrappers/Python/CoolProp/Plots

if __name__ == '__main__':
    import numpy, matplotlib
    from CoolProp.HumidAirProp import HAPropsSI
    from CoolProp.Plots.Plots import InlineLabel

    p = 101325
    Tdb = numpy.linspace(-10, 60, 100) + 273.15

    # Make the figure and the axes
    fig = matplotlib.pyplot.figure(figsize=(10, 8))
    ax = fig.add_axes((0.1, 0.1, 0.85, 0.85))

    # Saturation line
    w = [HAPropsSI('W', 'T', T, 'P', p, 'R', 1.0) for T in Tdb]
    ax.plot(Tdb - 273.15, w, lw=2)

    # Humidity lines
    RHValues = [0.05, 0.1, 0.15, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]
    for RH in RHValues:
        w = [HAPropsSI('W', 'T', T, 'P', p, 'R', RH) for T in Tdb]
        ax.plot(Tdb - 273.15, w, 'r', lw=1)

    # Humidity lines
    for H in [-20000, -10000, 0, 10000, 20000, 30000, 40000, 50000, 60000, 70000, 80000, 90000]:
        # Line goes from saturation to zero humidity ratio for this enthalpy
        T1 = HAPropsSI('T', 'H', H, 'P', p, 'R', 1.0) - 273.15
        T0 = HAPropsSI('T', 'H', H, 'P', p, 'R', 0.0) - 273.15
        w1 = HAPropsSI('W', 'H', H, 'P', p, 'R', 1.0)
        w0 = HAPropsSI('W', 'H', H, 'P', p, 'R', 0.0)
        ax.plot(numpy.r_[T1, T0], numpy.r_[w1, w0], 'r', lw=1)

    ax.set_xlim(Tdb[0] - 273.15, Tdb[-1] - 273.15)
    ax.set_ylim(0, 0.03)
    ax.set_xlabel(r"$T_{db}$ [$^{\circ}$C]")
    ax.set_ylabel(r"$W$ ($m_{w}/m_{da}$) [-]")

    xv = Tdb  # [K]
    for RH in [0.05, 0.1, 0.15, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]:
        yv = [HAPropsSI('W', 'T', T, 'P', p, 'R', RH) for T in Tdb]
        y = HAPropsSI('W', 'P', p, 'H', 65000.000000, 'R', RH)
        T_K, w, rot = InlineLabel(xv, yv, y=y, axis=ax)
        string = r'$\phi$=' + '{s:0.0f}'.format(s=RH * 100) + '%'
        bbox_opts = dict(boxstyle='square,pad=0.0', fc='white', ec='None', alpha=0.5)
        ax.text(T_K - 273.15, w, string, rotation=rot, ha='center', va='center', bbox=bbox_opts)

    matplotlib.pyplot.show()
